@@ -20,6 +20,25 @@ def test_host_network_mode(daemon):
         assert container.ip_address != '127.0.0.1'
 
 
+def test_name(daemon):
+    with ExitStack() as exit_stack:
+        name = 'foobar'
+        container = daemon.provision('alpine:3.10', name=name)
+        exit_stack.callback(container.remove)
+        assert container.name == name
+
+
+def test_environment(daemon):
+    with ExitStack() as exit_stack:
+        env_var = 'FOO'
+        env_val = 'FOOBARBAZ'
+        env = {env_var: env_val}
+        container = daemon.provision('alpine:3.10', environment=env)
+        exit_stack.callback(container.remove)
+        shell = container.shell()
+        assert shell.environ(env_var) == env_val
+
+
 def test_readonly_volume(daemon):
     with ExitStack() as exit_stack:
         expected = 'Hello!'
